## @class
# Search Filter encapsulation.
package DBCapsulator::SearchFilter;

use strict;
use DBCapsulator;
use DBI;

use base "DBCapsulator";
my $id = __PACKAGE__;


## @cmethod SearchFilter new()
# Create a new SearchFilter.
sub new {
    my( $this ) = @_;

    my  $class = ref($this) || $this;
    my $self = {};
    bless( $self, $class );
    return $self;
}


## @method $ sqlexpr($expr)
# Set or fetch the SQL expression that corresponds with this filter. This
# expression can be used as is in a SQL SELECT statement.
#
# @param expr String specifying the SQL expression. If this parameter is undefined
#             the current expression is returned without any change.
# @return The current search filter expression.
sub sqlexpr {
    my( $self, $selectexpr ) = @_;

    $self->{"$id sql"} = $selectexpr if( defined $selectexpr );
    return( $self->{"$id sql"} );
}


## @method void addorder($field,$descending)
# Add a field name to the list of fields the search result should be ordered by.
#
# @param field The name of the field that should be added.
# @param descending (optional) must be "1" if order should be descending.
sub addorder {
    my( $self, $field, $descending ) = @_;

    $self->{"$id order"} = my $order = $self->{"$id order"} || [];
    push( @$order, {
        "field" => $field,
	"descending" => $descending || 0,
    } );
}


## @method @ orderfields(@fields)
# Get or set a list of fields the search results should be ordered by.
#
# @param fields (optional) If set, the list of order field names is set to this list.
# @return The list of field names.
sub orderfields {
    my( $self, $fields ) = @_;

    if( defined($fields) ) {
        my @order = map { ("field" => $_, "descending" => 0) } @$fields;
	$self->{"$id order"} = \@order;
    }
    return() unless( defined $self->{"$id order"} );
    return( map { $_->{"field"} } @{$self->{"$id order"}} );
}


## @method boolean isdescending($field,$descending)
# Get or set if a field is to be sorted in descending order.
#
# @param field The name of the field.
# @param descending (optional) If set, the descending status is set to this value
# @return true if the field is to be ordered in descending order, false otherwise
sub isdescending {
    my( $self, $field, $descending ) = @_;

    my( $record ) = grep( $_->{"field"} eq $field, @{$self->{"$id order"}} );
    return unless( $record );
    $record->{"descending"} = $descending if( defined($descending) );
    return( $record->{"descending"} );
}


## @method $ sqlorder()
# Get the ordering settings as a string ready for inclusion in an SQL SELECT statement.
#
# @return String with an "ORDER BY" statement or an empty string if no order has been requested.
sub sqlorder {
    my( $self ) = @_;

    my @fields = $self->orderfields();
    return( "" ) unless(@fields);
    my $expr = "ORDER BY ";
    foreach my $field (@fields) {
        $expr .= "$field";
	$expr .= " DESC" if( $self->isdescending($field) );
	$expr .= ",";
    }
    return( substr($expr,0,length($expr)-1) );
}

1;
